@@ -1,4 +1,3 @@
-# $Id$
 #
 # BioPerl module for Bio::Tools::Run::Alignment::Clustalw
 #
@@ -205,7 +204,7 @@ clustalw program have not yet been implemented.  If you would like
 that a specific clustalw feature be added to this perl contact
 bioperl-l@bioperl.org.
 
-These can be specified as paramters when instantiating a new Clustalw
+These can be specified as parameters when instantiating a new Clustalw
 object, or through get/set methods of the same name (lowercase).
 
 =head1 PARAMETER FOR ALIGNMENT COMPUTATION
@@ -737,7 +736,8 @@ sub _run {
         
     	$self->debug( "Program ".$self->executable."\n");
     	my $commandstring = $self->executable."$instring"."$param_string";
-        $commandstring .= ' 1>/dev/null' if $quiet;
+        my $null = ($^O =~ m/mswin/i) ? 'NUL' : '/dev/null';
+        $commandstring .= " 1>$null" if $quiet;
         $self->debug( "clustal command = $commandstring");
         
         my $status = system($commandstring);
