@@ -1,4 +1,3 @@
-# $Id: Meme.pm,v 1.25 2006/07/04 22:23:31 mauricio Exp $
 # BioPerl module for Meme
 #
 # Please direct questions and support issues to <bioperl-l@bioperl.org> 
@@ -300,7 +299,9 @@ sub _setparams {
                                               -dash => 1);
     
     $param_string .= " > $outfile";
-    $param_string .= ' 2> /dev/null' if $self->quiet || $self->verbose < 0;
+
+    my $null = ($^O =~ m/mswin/i) ? 'NUL' : '/dev/null';
+    $param_string .= " 2> $null" if $self->quiet || $self->verbose < 0;
     
     return $param_string;
 }
