@@ -1,4 +1,3 @@
-# $Id$
 #
 # BioPerl module for Bio::Tools::Run::Phylo::PAML::Baseml
 #
@@ -554,7 +553,7 @@ sub set_parameter{
  Returns : none
  Args    : boolean: keep existing parameter values
  NB      : using this isn't an especially good idea! You don't need to do
-           anything to end up using default paramters: hence 'default'!
+           anything to end up using default parameters: hence 'default'!
 
 =cut
 
