@@ -1,7 +1,7 @@
 #
 # BioPerl module for Bio::Tools::Run::Phylo::Phyml
 #
-# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+# Please direct questions and support issues to <bioperl-l@bioperl.org>
 #
 # Cared for by Heikki Lehvaslaiho
 #
@@ -93,7 +93,10 @@ every script that will use this Phyml wrapper module, e.g.:
 
 =head2 Running
 
-This wrapper has been tested with PHYML v2.4.4 and v.3.0
+This wrapper has been tested with PHYML v2.4.4 and v.3.0.  It may work with
+recent Phyml releases using a date format for the format, but the wrapper
+hasn't been extensively tested in these cases, so for the moment only the
+simpler numbered versions are supported.
 
 In its current state, the wrapper supports only input of one MSA and
 output of one tree. It can easily be extended to support more advanced
@@ -115,15 +118,15 @@ the Bioperl mailing list.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
-=head2 Support 
+=head2 Support
 
 Please direct usage questions or support issues to the mailing list:
 
 I<bioperl-l@bioperl.org>
 
-rather than to the module maintainer directly. Many experienced and 
-reponsive experts will be able look at the problem and quickly 
-address it. Please include a thorough description of the problem 
+rather than to the module maintainer directly. Many experienced and
+reponsive experts will be able look at the problem and quickly
+address it. Please include a thorough description of the problem
 with code and data examples if at all possible.
 
 =head2 Reporting Bugs
@@ -132,7 +135,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 the web:
 
-  http://redmine.open-bio.org/projects/bioperl/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Heikki Lehvaslaiho
 
@@ -151,29 +154,30 @@ use strict;
 use Bio::AlignIO;
 use File::Copy;
 use File::Spec;
-
 use Bio::TreeIO;
 
 use base qw(Bio::Tools::Run::WrapperBase);
 
 our $PROGRAM_NAME = 'phyml';
-our $PROGRAM_DIR = $ENV{'PHYMLDIR'};
-
-
+our $PROGRAM_DIR  = $ENV{'PHYMLDIR'};
 
 # valid substitution model names
 our $models;
+
 # DNA
 map { $models->{0}->{$_} = 1 } qw(JC69 K2P F81 HKY F84 TN93 GTR);
+
 # protein
 map { $models->{1}->{$_} = 1 } qw(JTT MtREV Dayhoff WAG);
 
 our $models3;
+
 # DNA
 map { $models3->{'nt'}->{$_} = 1 } qw(HKY85 JC69 K80 F81 F84 TN93 GTR );
+
 # protein
 map { $models3->{'aa'}->{$_} = 1 }
-    qw(WAG JTT MtREV Dayhoff DCMut RtREV CpREV VT Blosum62 MtMam MtArt HIVw  HIVb );
+  qw(LG WAG JTT MtREV Dayhoff DCMut RtREV CpREV VT Blosum62 MtMam MtArt HIVw  HIVb );
 
 =head2 new
 
@@ -183,71 +187,88 @@ map { $models3->{'aa'}->{$_} = 1 }
  Returns : Bio::Tools::Run::Phylo::Phyml
  Args    : Optionally, provide any of the following (default in []):
            -data_type       => 'dna' or 'protein',   [protein]
-           -dataset_count   => 'integer,             [1]
+           -dataset_count   => integer,             [1]
            -model           => 'HKY'... ,            [HKY|JTT]
            -kappa           => 'e' or float,         [e]
            -invar           => 'e' or float,         [e]
            -category_number => integer,              [1]
            -alpha           => 'e' or float (int v3),[e]
            -tree            => 'BIONJ' or your own,  [BION]
-           -opt_topology    => boolean               [y]
-           -opt_lengths     => boolean               [y]
+           -bootstrap       => integer               [123]
+           -opt_topology    => boolean               [1]
+           -opt_lengths     => boolean               [1]
+           -no_memory_check => boolean               [1]
+           -program_name    => string
 
 =cut
 
 sub new {
-    my ($class, @args) = @_;
+    my ( $class, @args ) = @_;
     my $self = $class->SUPER::new(@args);
 
     # for consistency with other run modules, allow params to be dashless
     my %args = @args;
-    while (my ($key, $val) = each %args) {
-        if ($key !~ /^-/) {
+    while ( my ( $key, $val ) = each %args ) {
+        if ( $key !~ /^-/ ) {
             delete $args{$key};
-            $args{'-'.$key} = $val;
+            $args{ '-' . $key } = $val;
         }
     }
 
-    my ($data_type, $data_format, $dataset_count, $model, $freq, $kappa, $invar,
-	$category_number, $alpha, $tree, $opt_topology,
-	$opt_lengths, $opt, $search, $rand_start, $rand_starts, $rand_seed)
-        = $self->_rearrange([qw( DATA_TYPE
-				 DATA_FORMAT
-				 DATASET_COUNT
-				 MODEL
-				 FREQ
-				 KAPPA
-				 INVAR
-				 CATEGORY_NUMBER
-				 ALPHA
-				 TREE
-				 OPT_TOPOLOGY
-				 OPT_LENGTHS
-				 OPT
-				 SEARCH
-				 RAND_START
-				 RAND_STARTS
-				 RAND_SEED
-			      )], %args);
+    my (
+        $data_type, $data_format,     $dataset_count, $model,
+        $freq,      $kappa,           $invar,         $category_number,
+        $alpha,     $tree,            $opt_topology,  $opt_lengths,
+        $opt,       $search,          $rand_start,    $rand_starts,
+        $rand_seed, $no_memory_check, $bootstrap,     $program_name
+      )
+      = $self->_rearrange(
+        [
+            qw( DATA_TYPE
+              DATA_FORMAT
+              DATASET_COUNT
+              MODEL
+              FREQ
+              KAPPA
+              INVAR
+              CATEGORY_NUMBER
+              ALPHA
+              TREE
+              OPT_TOPOLOGY
+              OPT_LENGTHS
+              OPT
+              SEARCH
+              RAND_START
+              RAND_STARTS
+              RAND_SEED
+              NO_MEMORY_CHECK
+              BOOTSTRAP
+              PROGRAM_NAME
+              )
+        ],
+        %args
+      );
 
-    $self->data_type($data_type) if $data_type;
-    $self->data_format($data_format) if $data_format;
-    $self->dataset_count($dataset_count) if $dataset_count;
-    $self->model($model) if $model;
-    $self->freq($freq) if $freq;
-    $self->kappa($kappa) if $kappa;
-    $self->invar($invar) if $invar;
+    $self->data_type($data_type)             if $data_type;
+    $self->data_format($data_format)         if $data_format;
+    $self->dataset_count($dataset_count)     if $dataset_count;
+    $self->model($model)                     if $model;
+    $self->freq($freq)                       if $freq;
+    $self->kappa($kappa)                     if $kappa;
+    $self->invar($invar)                     if $invar;
     $self->category_number($category_number) if $category_number;
-    $self->alpha($alpha) if $alpha;
-    $self->tree($tree) if $tree;
-    $self->opt_topology($opt_topology) if $opt_topology;
-    $self->opt_lengths($opt_lengths) if $opt_lengths;
-    $self->opt($opt) if $opt;
-    $self->search($search) if $search;
-    $self->rand_start($rand_start) if $rand_start;
-    $self->rand_starts($rand_starts) if $rand_starts;
-    $self->rand_seed($rand_seed) if $rand_seed;
-
+    $self->alpha($alpha)                     if $alpha;
+    $self->tree($tree)                       if $tree;
+    $self->opt_topology($opt_topology)       if $opt_topology;
+    $self->opt_lengths($opt_lengths)         if $opt_lengths;
+    $self->opt($opt)                         if $opt;
+    $self->search($search)                   if $search;
+    $self->rand_start($rand_start)           if $rand_start;
+    $self->rand_starts($rand_starts)         if $rand_starts;
+    $self->rand_seed($rand_seed)             if $rand_seed;
+    $self->no_memory_check($no_memory_check) if $no_memory_check;
+    $self->bootstrap($bootstrap)             if $bootstrap;
+    $self->program_name($program_name)       if $program_name;
 
     return $self;
 }
@@ -263,7 +284,17 @@ sub new {
 =cut
 
 sub program_name {
-    return $PROGRAM_NAME;
+    my ( $self, $value ) = @_;
+
+    if ( defined($value) ) {
+      if ( $value =~ /^$PROGRAM_NAME[-a-z]*$/ ) {
+        $PROGRAM_NAME = $value;
+      } else {
+        $self->throw("$value is not a valid program name");
+      }
+    }
+
+    $PROGRAM_NAME;
 }
 
 =head2 program_dir
@@ -292,6 +323,9 @@ sub program_dir {
 Phyml before 3.0 did not display the version. Assume 2.44 when can not
 determine it.
 
+Some releases do not state version number, only date, so the
+version might have to be inferred from this date.
+
 =cut
 
 sub version {
@@ -299,40 +333,46 @@ sub version {
 
     return $self->{'_version'} if defined $self->{'_version'};
     my $exe = $self->executable || return;
-    my $string = substr `$exe --help`, 0, 40 ;
+    my $string = substr `$exe -h`, 0, 40;
     my ($version) = $string =~ /PhyML v([\d+\.]+)/;
-    $self->{'_version'} = $version;
-    $version ? (return $version) : return '2.44'
-}
+    if ( !$version ) {
+        $string =~ /PhyML\s+(\d{8})/;
 
+        # 3 was released August 2008
+        $version = 3 if ( $1 && $1 >= 20080801 );
+    }
+    $self->{'_version'} = $version;
+    $version ? ( return $version ) : return '2.44';
+}
 
 =head2 run
 
  Title   : run
  Usage   : $factory->run($aln_file);
            $factory->run($align_object);
- Function: Runs Phyml to generate a tree 
+ Function: Runs Phyml to generate a tree
  Returns : Bio::Tree::Tree object
- Args    : file name for your input alignment in a format 
+ Args    : file name for your input alignment in a format
            recognised by AlignIO, OR  Bio::Align::AlignI
-           complient object (eg. Bio::SimpleAlign).
+           compliant object (eg. Bio::SimpleAlign).
 
 =cut
 
 sub run {
-    my ($self, $in) = @_;
+    my ( $self, $in ) = @_;
 
-    if (ref $in && $in->isa("Bio::Align::AlignI")) {
+    if ( ref $in && $in->isa("Bio::Align::AlignI") ) {
         $in = $self->_write_phylip_align_file($in);
     }
-    elsif (! -e $in) {
-        $self->throw("When not supplying a Bio::Align::AlignI object, ".
-		     "you must supply a readable filename");
+    elsif ( !-e $in ) {
+        $self->throw( "When not supplying a Bio::Align::AlignI object, "
+              . "you must supply a readable filename" );
     }
-    elsif (-e $in) {
-	copy ($in, $self->tempdir);
-	my $name = File::Spec->splitpath($in); # name is the last item in the array
-	$in = File::Spec->catfile($self->tempdir, $name);
+    elsif ( -e $in ) {
+        copy( $in, $self->tempdir );
+        my $name =
+          File::Spec->splitpath($in);    # name is the last item in the array
+        $in = File::Spec->catfile( $self->tempdir, $name );
     }
 
     return $self->_run($in);
@@ -349,7 +389,7 @@ sub run {
 =cut
 
 sub stats {
-    my $self = shift;;
+    my $self = shift;
     return $self->{_stats};
 }
 
@@ -358,14 +398,14 @@ sub stats {
  Title   : tree_string
  Usage   : $factory->tree_string;
            $factory->run($align_object);
- Function: Returns the contents of the phyml '_phyml_tree.txt' ouput file
+ Function: Returns the contents of the phyml '_phyml_tree.txt' output file
  Returns : string with tree in Newick format, undef before run()
  Args    : none
 
 =cut
 
 sub tree_string {
-    my $self = shift;;
+    my $self = shift;
     return $self->{_tree};
 }
 
@@ -385,30 +425,32 @@ These methods are used to set and get program parameters before running.
 =cut
 
 sub data_type {
-    my ($self, $value) = @_;
-    if ($self->version && $self->version >= 3 ) {
-	if (defined $value) {
-	    if ($value eq 'nt') {
-		$self->{_data_type} = 'nt';
-	    } else {
-		$self->{_data_type} = 'aa';
-	    }
-	}
-	return 'aa' unless defined $self->{_data_type};
-    } else {
-	if (defined $value) {
-	    if ($value eq 'dna') {
-		$self->{_data_type} = '0';
-	    } else {
-		$self->{_data_type} = '1';
-	    }
-	}
-	return '1' unless defined $self->{_data_type};
+    my ( $self, $value ) = @_;
+    if ( $self->version && $self->version >= 3 ) {
+        if ( defined $value ) {
+            if ( $value eq 'nt' ) {
+                $self->{_data_type} = 'nt';
+            }
+            else {
+                $self->{_data_type} = 'aa';
+            }
+        }
+        return 'aa' unless defined $self->{_data_type};
+    }
+    else {
+        if ( defined $value ) {
+            if ( $value eq 'dna' ) {
+                $self->{_data_type} = '0';
+            }
+            else {
+                $self->{_data_type} = '1';
+            }
+        }
+        return '1' unless defined $self->{_data_type};
     }
     return $self->{_data_type};
 }
 
-
 =head2 data_format
 
  Title   : data_format
@@ -421,11 +463,12 @@ sub data_type {
 =cut
 
 sub data_format {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	$self->throw("PHYLIP format must be 'i' or 's'")
-	    unless $value eq 'i' or $value eq 's';
-	$self->{_data_format} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        $self->throw("PHYLIP format must be 'i' or 's'")
+          unless $value eq 'i'
+              or $value eq 's';
+        $self->{_data_format} = $value;
     }
     return $self->{_data_format} || 'i';
 }
@@ -441,17 +484,15 @@ sub data_format {
 =cut
 
 sub dataset_count {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid positive integer [$value]"
-	    unless $value =~ /^[-+]?\d*$/ and $value > 0;
-	$self->{_dataset_count} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid positive integer [$value]"
+          unless $value =~ /^[-+]?\d*$/ and $value > 0;
+        $self->{_dataset_count} = $value;
     }
     return $self->{_dataset_count} || 1;
 }
 
-
-
 =head2 model
 
  Title   : model
@@ -464,51 +505,55 @@ sub dataset_count {
            v3.0:
            HKY85 (default) | JC69 | K80 | F81 | F84 |
            TN93 | GTR (DNA)
-           WAG (default) | JTT | MtREV | Dayhoff | DCMut |
+           LG (default) | WAG | JTT | MtREV | Dayhoff | DCMut |
            RtREV | CpREV | VT | Blosum62 | MtMam | MtArt |
            HIVw |  HIVb (amino acids)
 
- Returns : Name of the model, defaults to {HKY|JTT}
+ Returns : Name of the model, v2.4.4 defaults to {HKY|JTT}
  Args    : None to get, string to set.
 
 =cut
 
 sub model {
-    my ($self, $value) = @_;
-    if (defined ($value)) {
-	if ($self->version && $self->version >= 3 ) {
-	    unless ($value =~ /\d{6}/) {
-		$self->throw("Not a valid model name [$value] for current data type (alphabet)")
-		unless $models3->{$self->data_type}->{$value};
-	    }
-	} else {
-	    $self->throw("Not a valid model name [$value] for current data type (alphabet)")
-	    unless $models->{$self->data_type}->{$value};
-	}
-	$self->{_model} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined($value) ) {
+        if ( $self->version && $self->version >= 3 ) {
+            unless ( $value =~ /\d{6}/ ) {
+                $self->throw(
+"Not a valid model name [$value] for current data type (alphabet)"
+                ) unless $models3->{ $self->data_type }->{$value};
+            }
+        }
+        else {
+            $self->throw(
+"Not a valid model name [$value] for current data type (alphabet)"
+            ) unless $models->{ $self->data_type }->{$value};
+        }
+        $self->{_model} = $value;
     }
 
-
-    if ($self->{_model}) {
-	return $self->{_model};
+    if ( $self->{_model} ) {
+        return $self->{_model};
     }
 
-    if ($self->version && $self->version >= 3 ) {
-	if ($self->data_type eq 'aa') {
-	    return 'WAG'; # protein
-	} else {
-	    return 'HKY85'; # DNA
-	}
-    } else {
-	if ($self->data_type) {
-	    return 'JTT'; # protein
-	} else {
-	    return 'HKY'; # DNA
-	}
+    if ( $self->version && $self->version >= 3 ) {
+        if ( $self->data_type eq 'aa' ) {
+            return 'LG';    # protein
+        }
+        else {
+            return 'HKY85';    # DNA
+        }
+    }
+    else {
+        if ( $self->data_type ) {
+            return 'JTT';      # protein
+        }
+        else {
+            return 'HKY';      # DNA
+        }
     }
 }
 
-
 =head2 kappa
 
  Title   : kappa
@@ -520,18 +565,18 @@ sub model {
 =cut
 
 sub kappa {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid number [$value]" 
-	    unless $value =~ /^[-+]?\d*\.?\d*$/ or $value eq 'e';
-	$self->{_kappa} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless $value =~ /^[-+]?\d*\.?\d*$/
+              or $value eq 'e';
+        $self->{_kappa} = $value;
     }
     return 'e' unless defined $self->{_kappa};
     return 'e' if $self->{_kappa} eq 'e';
-    return sprintf("%.1f", $self->{_kappa});
+    return sprintf( "%.1f", $self->{_kappa} );
 }
 
-
 =head2 invar
 
  Title   : invar
@@ -543,18 +588,18 @@ sub kappa {
 =cut
 
 sub invar {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid number [$value]" 
-	    unless $value =~ /^[-+]?\d*\.\d*$/ or $value eq 'e';
-	$self->{_invar} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless $value =~ /^[-+]?\d*\.\d*$/
+              or $value eq 'e';
+        $self->{_invar} = $value;
     }
     return 'e' unless defined $self->{_invar};
     return 'e' if $self->{_invar} eq 'e';
-    return sprintf("%.1f", $self->{_invar});
+    return sprintf( "%.1f", $self->{_invar} );
 }
 
-
 =head2 category_number
 
  Title   : category_number
@@ -566,17 +611,15 @@ sub invar {
 =cut
 
 sub category_number {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid postive integer [$value]" 
-	    unless $value =~ /^[+]?\d*$/ and $value > 0;
-	$self->{_category_number} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid postive integer [$value]"
+          unless $value =~ /^[+]?\d*$/ and $value > 0;
+        $self->{_category_number} = $value;
     }
     return $self->{_category_number} || 1;
 }
 
-
-
 =head2 alpha
 
  Title   : alpha
@@ -588,15 +631,16 @@ sub category_number {
 =cut
 
 sub alpha {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid number [$value]"
-	    unless $value =~ /^[-+]?\d*\.?\d*$/ or $value eq 'e';
-	$self->{_alpha} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless $value =~ /^[-+]?\d*\.?\d*$/
+              or $value eq 'e';
+        $self->{_alpha} = $value;
     }
     return 'e' unless defined $self->{_alpha};
     return 'e' if $self->{_alpha} eq 'e';
-    return sprintf("%.1f", $self->{_alpha}) || 'e';
+    return sprintf( "%.1f", $self->{_alpha} ) || 'e';
 }
 
 =head2 tree
@@ -609,13 +653,12 @@ sub alpha {
 
 =cut
 
-
 sub tree {
-    my ($self, $value) = @_;
-    if (defined $value) {
-	die "Invalid number [$value]" 
-	    unless -e $value or $value eq 'BIONJ';
-	$self->{_tree} = $value;
+    my ( $self, $value ) = @_;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless -e $value or $value eq 'BIONJ';
+        $self->{_tree} = $value;
     }
     return $self->{_tree} || 'BIONJ';
 }
@@ -629,7 +672,7 @@ These methods can be used with PhyML v2* only.
  Title   : opt_topology
  Usage   : $factory->opt_topology(1);
  Function: Choose to optimise the tree topology
- Returns : {y|n} (default y)
+ Returns : 1 or 0. Default is 1.
  Args    : None to get, boolean to set.
 
 v2.* only
@@ -637,24 +680,26 @@ v2.* only
 =cut
 
 sub opt_topology {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [opt_topology] for to PhyML v3") if $self->version && $self->version >= 3;
-    if (defined ($value)) {
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [opt_topology] for to PhyML v3")
+      if $self->version && $self->version >= 3;
+    if ( defined($value) ) {
         if ($value) {
-	    $self->{_opt_topology} = 'y';
-	} else {
-	    $self->{_opt_topology} = 'n';
-	}
+            $self->{_opt_topology} = 1;
+        }
+        else {
+            $self->{_opt_topology} = 0;
+        }
     }
-    return $self->{_opt_topology} || 'y';
+    return $self->{_opt_topology} || 1;
 }
 
 =head2 opt_lengths
 
  Title   : opt_lengths
  Usage   : $factory->opt_lengths(0);
- Function: Choose to  optimise branch lengths and rate parameters 
- Returns : {y|n} (default y)
+ Function: Choose to  optimise branch lengths and rate parameters
+ Returns : 1 or 0. Default is 1.
  Args    : None to get, boolean to set.
 
 
@@ -663,16 +708,18 @@ v2.* only
 =cut
 
 sub opt_lengths {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [opt_lengths] for PhyML v3") if $self->version && $self->version >= 3;
-    if (defined ($value)) {
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [opt_lengths] for PhyML v3")
+      if $self->version && $self->version >= 3;
+    if ( defined($value) ) {
         if ($value) {
-	    $self->{_opt_lengths} = 'y';
-	} else {
-	    $self->{_opt_lengths} = 'n';
-	}
+            $self->{_opt_lengths} = 1;
+        }
+        else {
+            $self->{_opt_lengths} = 0;
+        }
     }
-    return $self->{_opt_lengths} || 'y';
+    return $self->{_opt_lengths} || 1;
 }
 
 =head2 v3 options
@@ -693,12 +740,15 @@ v3 only.
 =cut
 
 sub freq {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [freq] prior to PhyML v3") if $self->version < 3;
-    if (defined $value) {
-	die "Invalid value [$value]"
-	    unless $value =~ /^[\d\. ]$/ or $value eq 'e' or $value eq 'd';
-	$self->{_freq} = $value;
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [freq] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined $value ) {
+        die "Invalid value [$value]"
+          unless $value =~ /^[\d\. ]$/
+              or $value eq 'e'
+              or $value eq 'd';
+        $self->{_freq} = $value;
     }
     return $self->{_freq};
 }
@@ -716,10 +766,11 @@ v3.* only
 =cut
 
 sub opt {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [opt] prior to PhyML v3") if $self->version < 3;
-    if (defined ($value)) {
-	$self->{_opt} = $value if $value =~ /tlr|tl|tr|l|n/;
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [opt] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined($value) ) {
+        $self->{_opt} = $value if $value =~ /tlr|tl|tr|l|n/;
     }
     return $self->{_opt} || 'n';
 }
@@ -737,10 +788,11 @@ v3.* only
 =cut
 
 sub search {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [search] prior to PhyML v3") if $self->version < 3;
-    if (defined ($value)) {
-	$self->{_search} = $value if $value =~ /NNI|SPR|BEST/;
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [search] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined($value) ) {
+        $self->{_search} = $value if $value =~ /NNI|SPR|BEST/;
     }
     return $self->{_search} || 'NNI';
 }
@@ -757,21 +809,21 @@ v3.* only; only meaningful if $prog-E<gt>search is 'SPR'
 
 =cut
 
-
 sub rand_start {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [rand_start] prior to PhyML v3") if $self->version < 3;
-    if (defined ($value)) {
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [rand_start] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined($value) ) {
         if ($value) {
-	    $self->{_rand_start} = 1;
-	} else {
-	    $self->{_rand_start} = 0;
-	}
+            $self->{_rand_start} = 1;
+        }
+        else {
+            $self->{_rand_start} = 0;
+        }
     }
-    return $self->{_rand_start} || 0;
+    return $self->{_rand_start};
 }
 
-
 =head2 rand_starts
 
  Title   : rand_starts
@@ -785,17 +837,17 @@ v3.* only; only valid if $prog-E<gt>search is 'SPR'
 =cut
 
 sub rand_starts {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [rand_starts] prior to PhyML v3") if $self->version < 3;
-    if (defined $value) {
-	die "Invalid number [$value]"
-	    unless $value =~ /^[-+]?\d+$/;
-	$self->{_rand_starts} = $value;
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [rand_starts] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless $value =~ /^[-+]?\d+$/;
+        $self->{_rand_starts} = $value;
     }
     return $self->{_rand_starts} || 1;
 }
 
-
 =head2 rand_seed
 
  Title   : rand_seed
@@ -811,16 +863,85 @@ Uses perl rand() to initialize if not explicitely set.
 =cut
 
 sub rand_seed {
-    my ($self, $value) = @_;
-    $self->throw("Not a valid parameter [rand_seed] prior to PhyML v3") if $self->version < 3;
-    if (defined $value) {
-	die "Invalid number [$value]"
-	    unless $value =~ /^[-+]?\d+$/;
-	$self->{_rand_seed} = $value;
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [rand_seed] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined $value ) {
+        die "Invalid number [$value]"
+          unless $value =~ /^[-+]?\d+$/;
+        $self->{_rand_seed} = $value;
     }
     return $self->{_rand_seed} || int rand 1000000;
 }
 
+=head2 no_memory_check
+
+ Title   : no_memory_check
+ Usage   : $factory->no_memory_check(1);
+ Function:
+ Returns : boolean (defaults to false)
+ Args    : None to get, integer to set.
+
+=cut
+
+sub no_memory_check {
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [no_memory_check] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined($value) ) {
+        if ($value) {
+            $self->{_no_memory_check} = 1;
+        }
+        else {
+            $self->{_no_memory_check} = 0;
+        }
+    }
+    return $self->{_no_memory_check} || 0;
+}
+
+=head2 bootstrap
+
+ Title   : bootstrap
+ Usage   : $factory->bootstrap(100);
+ Function: Set number of bootstraps
+ Returns :
+ Args    : None to get, integer to set.
+
+=cut
+
+sub bootstrap {
+    my ( $self, $value ) = @_;
+    $self->throw("Not a valid parameter [bootstrap] prior to PhyML v3")
+      if $self->version < 3;
+    if ( defined $value ) {
+        die "Invalid number [$value]" unless $value =~ /^\d+$/;
+        $self->{_bootstrap} = $value;
+    }
+    return $self->{_bootstrap};
+}
+
+=head2 command
+
+ Title   : command
+ Usage   : $factory->command(...);
+ Function:
+ Returns : string
+ Args    : None to get, integer to set.
+
+=cut
+
+sub command {
+    my ( $self, $value ) = @_;
+    if ( defined($value) ) {
+        if ($value =~ /$PROGRAM_NAME/ ) {
+            $self->{_command} = $value;
+        }
+        else {
+            $self->throw("$value is not a $PROGRAM_NAME command");
+        }
+    }
+    return $self->{_command} || '';
+}
 
 =head2 Internal methods
 
@@ -829,45 +950,50 @@ These methods are private and should not be called outside this class.
 =cut
 
 sub _run {
-    my ($self, $file)= @_;
+    my ( $self, $file ) = @_;
 
     my $exe = $self->executable || return;
+
     my $command;
     my $output_stat_file;
-    if ($self->version >= 3 ) {
-	$command = $exe. " -i $file". $self->_setparams;
-	$output_stat_file = '_phyml_stats.txt';
-    } else {
-	$command = $exe. " $file ". $self->arguments. $self->_setparams;
-	$output_stat_file = '_phyml_stat.txt';
+    if ( $self->version >= 3 ) {
+        $command          = $exe . " -i $file" . $self->_setparams;
+        $output_stat_file = '_phyml_stats.txt';
+    }
+    else {
+        $command = $exe . " $file " . $self->arguments . $self->_setparams;
+        $output_stat_file = '_phyml_stat.txt';
     }
 
+    $self->command($command);
     $self->debug("Phyml command = $command\n");
     `$command`;
 
     # stats
     {
-	my $stat_file =  $file. $output_stat_file;
-	open(my $FH_STAT, "<", $stat_file)
-	    || $self->throw("Phyml call ($command) did not give an output [$stat_file]: $?");
-	local $/;
-	$self->{_stats} .= <$FH_STAT>;
+        my $stat_file = $file . $output_stat_file;
+        open( my $FH_STAT, "<", $stat_file )
+          || $self->throw(
+            "Phyml call ($command) did not give an output [$stat_file]: $?");
+        local $/;
+        $self->{_stats} .= <$FH_STAT>;
     }
+
     #print $self->{stats};
 
     # tree
-    my $tree_file =  $file. '_phyml_tree.txt';
+    my $tree_file = $file . '_phyml_tree.txt';
     {
-	open(my $FH_TREE, "<", $tree_file)
-	    || $self->throw("Phyml call ($command) did not give an output: $?");
-	local $/;
-	$self->{_tree} .= <$FH_TREE>;
+        open( my $FH_TREE, "<", $tree_file )
+          || $self->throw("Phyml call ($command) did not give an output: $?");
+        local $/;
+        $self->{_tree} .= <$FH_TREE>;
     }
 
-    open(my $FH_TREE, "<", $tree_file)
-	|| $self->throw("Phyml call ($command) did not give an output: $?");
+    open( my $FH_TREE, "<", $tree_file )
+      || $self->throw("Phyml call ($command) did not give an output: $?");
 
-    my $treeio = Bio::TreeIO->new(-format => 'nhx', -fh => $FH_TREE);
+    my $treeio = Bio::TreeIO->new( -format => 'nhx', -fh => $FH_TREE );
     my $tree = $treeio->next_tree;
 
     # could be faster to parse the tree only if needed?
@@ -875,14 +1001,13 @@ sub _run {
     return $tree;
 }
 
-
 =head2 _setparams
 
  Title   : _setparams
  Usage   : Internal function, not to be called directly
  Function: Creates a string of params to be used in the command string
  Returns : string of params
- Args    : none 
+ Args    : none
 
 =cut
 
@@ -890,56 +1015,61 @@ sub _setparams {
     my $self = shift;
     my $param_string;
 
-    if ($self->version >= 3 ) {
+    if ( $self->version >= 3 ) {
 
-	$param_string = ' -d '.  $self->data_type;
-	$param_string .= ' -q' if $self->data_format eq 's';
-	$param_string .= ' -n '. $self->dataset_count if $self->dataset_count > 1;
+        # version 3 or higher
+        $param_string = ' -d ' . $self->data_type;
 
-	#$param_string .= ' 0';	# no bootstap sets
+        $param_string .= ' -q ' if $self->data_format eq 's';
+        $param_string .= ' -n ' . $self->dataset_count
+          if $self->dataset_count > 1;
+        $param_string .= ' -b ' . $self->bootstrap if $self->bootstrap;
 
-	$param_string .= ' -m '. $self->model;
-	$param_string .= ' -f '. $self->freq if $self->freq;
+        # $param_string .= ' 0';  # no bootstrap sets
+        $param_string .= ' -m ' . $self->model;
+        $param_string .= ' -f ' . $self->freq if $self->freq;
 
-	if ($self->data_type eq 'dna') {
-	    $param_string .= ' -t '. $self->kappa;
-	}
+        if ( $self->data_type eq 'dna' ) {
+            $param_string .= ' -t ' . $self->kappa;
+        }
 
-	$param_string .= ' -v '. $self->invar;
-	$param_string .= ' -c '. $self->category_number;
-	$param_string .= ' -a '. $self->alpha;
-	$param_string .= ' -u '. $self->tree if $self->tree ne 'BIONJ';
-	$param_string .= ' -o '. $self->opt if $self->opt;
-	$param_string .= ' -s '. $self->search;
+        $param_string .= ' -v ' . $self->invar;
+        $param_string .= ' -c ' . $self->category_number;
+        $param_string .= ' -a ' . $self->alpha;
+        $param_string .= ' -u ' . $self->tree if $self->tree ne 'BIONJ';
+        $param_string .= ' -o ' . $self->opt if $self->opt;
+        $param_string .= ' -s ' . $self->search;
 
-	if ($self->search eq 'SPR' ) {
-	    $param_string .= ' --rand_start ' if $self->rand_start;
-	    $param_string .= ' --n_rand_starts '. $self->rand_starts
-	        if $self->rand_starts;
-	    $param_string .= ' --r_seed '. $self->rand_seed;
-	}
+        if ( $self->search eq 'SPR' ) {
+            $param_string .= ' --rand_start ' if $self->rand_start;
+            $param_string .= ' --n_rand_starts ' . $self->rand_starts
+              if $self->rand_starts;
+            $param_string .= ' --r_seed ' . $self->rand_seed;
+        }
 
-    } else {
+        $param_string .= ' --no_memory_check '
+          if $self->no_memory_check;
 
-	$param_string = ' ' .  $self->data_type;
+    }
+    else {
 
-	$param_string .= ' '. $self->data_format;
-	$param_string .= ' '. $self->dataset_count;
+        # version 2
+        $param_string = ' ' . $self->data_type;
+        $param_string .= ' ' . $self->data_format;
+        $param_string .= ' ' . $self->dataset_count;
+        $param_string .= ' 0';                         # no bootstrap sets
+        $param_string .= ' ' . $self->model;
 
-	$param_string .= ' 0';	# no bootstap sets
+        unless ( $self->data_type ) {                  # only for DNA
+            $param_string .= ' ' . $self->kappa;
+        }
 
-	$param_string .= ' '. $self->model;
-
-	unless ($self->data_type) { # only for DNA
-	    $param_string .= ' '. $self->kappa;
-	}
-
-	$param_string .= ' '. $self->invar;
-	$param_string .= ' '. $self->category_number;
-	$param_string .= ' '. $self->alpha;
-	$param_string .= ' '. $self->tree;
-	$param_string .= ' '. $self->opt_topology;
-	$param_string .= ' '. $self->opt_lengths;
+        $param_string .= ' ' . $self->invar;
+        $param_string .= ' ' . $self->category_number;
+        $param_string .= ' ' . $self->alpha;
+        $param_string .= ' ' . $self->tree;
+        $param_string .= ' ' . $self->opt_topology;
+        $param_string .= ' ' . $self->opt_lengths;
 
     }
     return $param_string;
@@ -951,7 +1081,7 @@ sub _setparams {
  Usage   : obj->__write_phylip_align_file($aln)
  Function: Internal (not to be used directly)
 
-           Writes the alignment into the tmp directory 
+           Writes the alignment into the tmp directory
            in PHYLIP interlieved format
 
  Returns : filename
@@ -960,14 +1090,16 @@ sub _setparams {
 =cut
 
 sub _write_phylip_align_file {
-    my ($self, $align) = @_;
-    
-    my $tempfile = File::Spec->catfile($self->tempdir, "aln$$.phylip");
+    my ( $self, $align ) = @_;
+
+    my $tempfile = File::Spec->catfile( $self->tempdir, "aln$$.phylip" );
     $self->data_format('i');
-    my $out = Bio::AlignIO->new('-file'     => ">$tempfile", 
-				'-format' => 'phylip',
-				'-interleaved' => 0,
-				'-longid' => 1 );
+    my $out = Bio::AlignIO->new(
+        -file        => ">$tempfile",
+        -format      => 'phylip',
+        -interleaved => 1,
+        -longid      => 1
+    );
     $out->write_aln($align);
     $out->close();
     $out = undef;
